@@ -7,7 +7,7 @@
  *
  *              More mods Robert Bond, 12/86
  *
- *		$Revision: 6.21 $
+ *		$Revision: 7.1 $
  */
 
 #include <sys/types.h>
@@ -28,18 +28,11 @@
 #include "sc.h"
 #include <signal.h>
 #include <errno.h>
+#include <stdlib.h>
 
-#ifdef SYSV3
-extern void exit();
-#else
-extern int exit();
-#endif
-
-void	openrow();
-void	syncref();
-void	unspecial();
-
-extern	int	errno;
+void	openrow(int rs);
+void	syncref(register struct enode *e);
+void	unspecial(FILE *f, char *str, int delim);
 
 /* a linked list of free [struct ent]'s, uses .next as the pointer */
 extern	struct ent *freeents;
@@ -87,12 +80,12 @@ dupcol()
     }
     modflg++;
     curcol++;
-    opencol (curcol, 1);
+    opencol(curcol, 1);
     for (currow = 0; currow <= maxrow; currow++) {
 	register struct ent *p = *ATBL(tbl, currow, curcol - 1);
 	if (p) {
 	    register struct ent *n;
-	    n = lookat (currow, curcol);
+	    n = lookat(currow, curcol);
 	    copyent ( n, p, 0, 1);
 	}
     }
@@ -107,31 +100,38 @@ dupcol()
 
 /* insert 'arg' rows before currow */
 void
-insertrow(arg)
-register int arg;
+insertrow(register int arg)
 {
-    while (--arg>=0) openrow (currow);
+    while (--arg>=0) openrow(currow);
 }
 
 /* delete 'arg' rows starting at currow (deletes from currow downward) */
 void
-deleterow(arg)
-register int arg;
+deleterow(register int arg)
 {
+    int rs = maxrow - currow + 1;
+    char buf[50];
+
+    if (rs - arg < 0)
+    {	rs = rs > 0 ? rs : 0;
+	(void) sprintf(buf, "Can't delete %d row%s %d row%s left", arg,
+		(arg != 1 ? "s," : ","), rs, (rs != 1 ? "s" : ""));
+	error(buf);
+	return;
+    }
     if (any_locked_cells(currow, 0, currow + arg - 1, maxcol))
 	error("Locked cells encountered. Nothing changed");
     else {
     	flush_saved();
     	erase_area(currow, 0, currow + arg - 1, maxcol);
     	currow += arg;
-    	while (--arg>=0) closerow (--currow);
+    	while (--arg>=0) closerow(--currow);
     	sync_refs();
     }
 }
 
 void
-erase_area(sr, sc, er, ec)
-int sr, sc, er, ec;
+erase_area(int sr, int sc, int er, int ec)
 {
     register int r, c;
     register struct ent **pp;
@@ -166,8 +166,7 @@ int sr, sc, er, ec;
  * containing whatever was on the screen
  */
 void
-valueize_area(sr, sc, er, ec)
-int sr, sc, er, ec;
+valueize_area(int sr, int sc, int er, int ec)
 {
     register int r, c;
     register struct ent *p;
@@ -204,8 +203,7 @@ int sr, sc, er, ec;
 }
 
 void
-pullcells(to_insert)
-int to_insert;
+pullcells(int to_insert)
 {
     register struct ent *p, *n;
     register int deltar, deltac;
@@ -307,8 +305,7 @@ rowshow_op()
  */
 
 int
-get_rcqual (ch)
-    int ch;
+get_rcqual(int ch)
 {
     error ("%sow/column:  r: row  c: column%s",
 
@@ -349,10 +346,9 @@ get_rcqual (ch)
 }
 
 void
-openrow (rs)
-int	rs;
+openrow(int rs)
 {
-    register	r, c;
+    register	r, c, i;
     struct ent	**tmprow, **pp;
 
     if (rs > maxrow) maxrow = rs;
@@ -373,23 +369,30 @@ int	rs;
 		if (*pp)
 			(*pp)->row = r;
     }
-    tbl[r] = tmprow;	/* the last row was never used.... */
+    tbl[r] = tmprow;		/* the last row was never used.... */
+
+    for (i=0; i < 26; i++) {	/* update all marked cells */
+	if (savedrow[i] >= rs)
+	    savedrow[i]++;
+    }
     FullUpdate++;
     modflg++;
 }
 
 /* delete row r */
 void
-closerow (r)
-register r;
+closerow(register r)
 {
     register struct ent **pp;
-    register c;
-    struct ent	**tmprow;
+    register		c;
+    int			saver;
+    struct ent		**tmprow;
+    register		i;
 
     if (r > maxrow) return;
 
     /* save the row and empty it out */
+    saver = r;
     tmprow = tbl[r];
     pp = ATBL(tbl, r, 0);
     for (c=maxcol+1; --c>=0; pp++) {
@@ -410,15 +413,24 @@ register r;
     }
     tbl[r] = tmprow;
 
+    /* restore r and update all marked cells */
+    r = saver;
+    for (i=0; i < 26; i++) {
+	if (savedrow[i] == r) {
+	    savedrow[i] = -1;
+	    savedcol[i] = -1;
+	}
+	if (savedrow[i] > r)
+	    savedrow[i]--;
+    }
+
     maxrow--;
     FullUpdate++;
     modflg++;
 }
 
 void
-opencol (cs, numcol)
-int	cs;
-int	numcol;
+opencol(int cs, int numcol)
 {
     register r;
     register struct ent **pp;
@@ -455,27 +467,30 @@ int	numcol;
 	for (c = cs; c - cs < numcol; c++, pp++)
 		*pp = (struct ent *)0;
     }
+    for (i=0; i < 26; i++) {	/* update all marked cells */
+	if (savedcol[i] >= cs)
+	    savedcol[i] += numcol;
+    }
     FullUpdate++;
     modflg++;
 }
 
 /* delete group of columns (1 or more) */
 void
-closecol (cs, numcol)
-int cs;
-int	numcol;
+closecol(int cs, int numcol)
 {
     register r;
     register struct ent **pp;
     register struct ent *q;
     register c;
-    register lim = maxcol-cs;
+    register lim = maxcol - cs + 1;
     int i;
     char buf[50];
 
-    if (lim - numcol < -1)
-    {	(void) sprintf(buf, "Can't delete %d column%s %d columns left", numcol,
-			(numcol > 1 ? "s," : ","), lim+1);
+    if (lim - numcol < 0)
+    {	lim = lim > 0 ? lim : 0;
+	(void) sprintf(buf, "Can't delete %d column%s %d column%s left", numcol,
+		(numcol != 1 ? "s," : ","), lim, (lim != 1 ? "s" : ""));
 	error(buf);
 	return;
     }
@@ -521,14 +536,22 @@ int	numcol;
 	col_hidden[i] = FALSE;
     }
 
+    for (i=0; i < 26; i++) {	/* update all marked cells */
+	if (savedcol[i] >= cs && savedcol[i] < cs+numcol) {
+	    savedrow[i] = -1;
+	    savedcol[i] = -1;
+	}
+	if (savedcol[i] > cs)
+	    savedcol[i] -= numcol;
+    }
+
     maxcol -= numcol;
     FullUpdate++;
     modflg++;
 }
 
 void
-doend(rowinc, colinc)
-int rowinc, colinc;
+doend(int rowinc, int colinc)
 {
     register struct ent *p;
     int r, c;
@@ -602,8 +625,7 @@ int rowinc, colinc;
 
 /* Modified 9/17/90 THA to handle more formats */
 void
-doformat(c1,c2,w,p,r)
-int c1,c2,w,p,r;
+doformat(int c1, int c2, int w, int p, int r)
 {
     register int i;
     int crows = 0;
@@ -636,11 +658,11 @@ int c1,c2,w,p,r;
 }
 
 void
-print_options(f)
-FILE *f;
+print_options(FILE *f)
 {
     if(
        autocalc &&
+       !optimize &&
        propagation == 10 &&
        calc_order == BYROWS &&
        !numeric &&
@@ -658,6 +680,8 @@ FILE *f;
     (void) fprintf(f, "set");
     if(!autocalc) 
 	(void) fprintf(f," !autocalc");
+    if(optimize) 
+	(void) fprintf(f," optimize");
     if(propagation != 10)
 	(void) fprintf(f, " iterations = %d", propagation);
     if(calc_order != BYROWS )
@@ -688,9 +712,7 @@ FILE *f;
 }
 
 void
-printfile (fname, r0, c0, rn, cn)
-char *fname;
-int r0, c0, rn, cn;
+printfile(char *fname, int r0, int c0, int rn, int cn)
 {
     FILE *f;
     static char *pline = NULL;		/* only malloc once, malloc is slow */
@@ -877,9 +899,7 @@ int r0, c0, rn, cn;
 }
 
 void
-tblprintfile (fname, r0, c0, rn, cn)
-char *fname;
-int r0, c0, rn, cn;
+tblprintfile(char *fname, int r0, int c0, int rn, int cn)
 {
     FILE *f;
     int pid;
@@ -1060,10 +1080,7 @@ int r0, c0, rn, cn;
 
 /* unspecial (backquote) things that are special chars in a table */
 void
-unspecial(f, str, delim)
-FILE	*f;
-char	*str;
-int	delim;
+unspecial(FILE *f, char *str, int delim)
 {
 	if( *str == '\\' ) str++; /* delete wheeling string operator, OK? */
 	while (*str)
@@ -1079,9 +1096,7 @@ int	delim;
 }
 
 struct enode *
-copye (e, Rdelta, Cdelta)
-register struct enode *e;
-int Rdelta, Cdelta;
+copye(register struct enode *e, int Rdelta, int Cdelta)
 {
     register struct enode *ret;
 
@@ -1167,8 +1182,7 @@ sync_refs ()
 }
 
 void
-syncref(e)
-register struct enode *e;
+syncref(register struct enode *e)
 {
     if (e == (struct enode *)0)
 	return;
@@ -1194,8 +1208,7 @@ register struct enode *e;
 
 /* mark a row as hidden */
 void
-hiderow(arg)
-int arg;
+hiderow(int arg)
 {
     register int r1;
     register int r2;
@@ -1220,8 +1233,7 @@ int arg;
 
 /* mark a column as hidden */
 void
-hidecol(arg)
-int arg;
+hidecol(int arg)
 {
     register int c1;
     register int c2;
@@ -1246,8 +1258,7 @@ int arg;
 
 /* mark a row as not-hidden */
 void
-showrow(r1, r2)
-int r1, r2;
+showrow(int r1, int r2)
 {
     if (r1 < 0 || r1 > r2) {
 	error ("Invalid range");
@@ -1264,8 +1275,7 @@ int r1, r2;
 
 /* mark a column as not-hidden */
 void
-showcol(c1, c2)
-int c1, c2;
+showcol(int c1, int c2)
 {
     if (c1 < 0 || c1 > c2) {
 	error ("Invalid range");
@@ -1282,9 +1292,7 @@ int c1, c2;
 
 /* Open the output file, setting up a pipe if needed */
 FILE *
-openout(fname, rpid)
-char *fname;
-int *rpid;
+openout(char *fname, int *rpid)
 {
     int pipefd[2];
     int pid;
@@ -1350,9 +1358,7 @@ int *rpid;
 
 /* close a file opened by openout(), if process wait for return */
 void
-closeout(f, pid)
-FILE *f;
-int pid;
+closeout(FILE *f, int pid)
 {
     int temp;
 
@@ -1369,9 +1375,7 @@ int pid;
 }
 
 void
-copyent(n,p,dr,dc)
-	    register struct ent *n, *p;
-	    int dr, dc;
+copyent(register struct ent *n, register struct ent *p, int dr, int dc)
 {
     if(!n||!p){error("internal error");return;}
     n -> v = p -> v;
@@ -1390,9 +1394,7 @@ copyent(n,p,dr,dc)
 }
 
 void
-write_fd (f, r0, c0, rn, cn)
-register FILE *f;
-int r0, c0, rn, cn;
+write_fd(register FILE *f, int r0, int c0, int rn, int cn)
 {
     register struct ent **pp;
     register r, c;
@@ -1447,9 +1449,7 @@ int r0, c0, rn, cn;
 }
 
 int
-writefile (fname, r0, c0, rn, cn)
-char *fname;
-int r0, c0, rn, cn;
+writefile(char *fname, int r0, int c0, int rn, int cn)
 {
     register FILE *f;
     char save[PATHLEN];
@@ -1484,9 +1484,7 @@ int r0, c0, rn, cn;
 }
 
 void
-readfile (fname,eraseflg)
-char *fname;
-int eraseflg;
+readfile(char *fname, int eraseflg)
 {
     register FILE *f;
     char save[PATHLEN];
@@ -1519,12 +1517,12 @@ int eraseflg;
 	return;
     }
 
-    if (eraseflg) erasedb ();
+    if (eraseflg) erasedb();
 
     loading++;
     while (fgets(line, sizeof(line), f)) {
 	linelim = 0;
-	if (line[0] != '#') (void) yyparse ();
+	if (line[0] != '#') (void) yyparse();
     }
     --loading;
     (void) fclose (f);
@@ -1540,7 +1538,7 @@ int eraseflg;
 
 /* erase the database (tbl, etc.) */
 void
-erasedb ()
+erasedb()
 {
     register r, c;
     for (c = 0; c<=maxcol; c++) {
@@ -1553,8 +1551,8 @@ erasedb ()
 	register struct ent **pp = ATBL(tbl, r, 0);
 	for (c=0; c++<=maxcol; pp++)
 	    if (*pp) {
-		if ((*pp)->expr)  efree ((*pp) -> expr);
-		if ((*pp)->label) scxfree ((char *)((*pp) -> label));
+		if ((*pp)->expr)  efree((*pp) -> expr);
+		if ((*pp)->label) scxfree((char *)((*pp) -> label));
 		(*pp)->next = freeents;	/* save [struct ent] for reuse */
 		freeents = *pp;
 		*pp = (struct ent *)0;
@@ -1568,8 +1566,7 @@ erasedb ()
 
 /* moves curcol back one displayed column */
 void
-backcol(arg)
-	int arg;
+backcol(int arg)
 {
     while (--arg>=0) {
 	if (curcol)
@@ -1583,8 +1580,7 @@ backcol(arg)
 
 /* moves curcol forward one displayed column */
 void
-forwcol(arg)
-	int arg;
+forwcol(int arg)
 {
     while (--arg>=0) {
 	if (curcol < maxcols - 1)
@@ -1594,15 +1590,14 @@ forwcol(arg)
 		break;
 	else
 		curcol++;
-	while(col_hidden[curcol]&&(curcol<maxcols-1))
+	while (col_hidden[curcol]&&(curcol<maxcols-1))
 	    curcol++;
     }
 }
 
 /* moves currow forward one displayed row */
 void
-forwrow(arg)
-	int arg;
+forwrow(int arg)
 {
     while (--arg>=0) {
 	if (currow < maxrows - 1)
@@ -1619,14 +1614,13 @@ forwrow(arg)
 
 /* moves currow backward one displayed row */
 void
-backrow(arg)
-	int arg;
+backrow(int arg)
 {
     while (--arg>=0) {
 	if (currow)
 	    currow--;
 	else
-	    {error ("At row zero"); break;}
+	    {error("At row zero"); break;}
 	while (row_hidden[currow] && currow)
 	    currow--;
     }
@@ -1640,15 +1634,14 @@ backrow(arg)
  */
 
 void
-showstring (string, dirflush, hasvalue, row, col, nextcolp, mxcol, fieldlenp, r, c)
-    char *string;	/* to display */
-    int dirflush;	/* or rightflush or centered */
-    int hasvalue;	/* is there a numeric value? */
-    int row, col;	/* spreadsheet location */
-    int *nextcolp;	/* value returned through it */
-    int mxcol;		/* last column displayed? */
-    int *fieldlenp;	/* value returned through it */
-    int r, c;		/* screen row and column */
+showstring(char *string,	/* to display */
+    int dirflush,		/* or rightflush or centered */
+    int hasvalue,		/* is there a numeric value? */
+    int row, int col,		/* spreadsheet location */
+    int *nextcolp,		/* value returned through it */
+    int mxcol,			/* last column displayed? */
+    int *fieldlenp,		/* value returned through it */
+    int r, int c)		/* screen row and column */
 {
     register int nextcol  = *nextcolp;
     register int fieldlen = *fieldlenp;
@@ -1662,8 +1655,8 @@ showstring (string, dirflush, hasvalue, row, col, nextcolp, mxcol, fieldlenp, r,
     /* This figures out if the label is allowed to
        slop over into the next blank field */
 
-    slen = strlen (string);
-    if( *string == '\\' && *(string+1)!= '\0' )
+    slen = strlen(string);
+    if (*string == '\\' && *(string+1)!= '\0')
 	slen = fwidth[col];
     while ((slen > fieldlen) && (nextcol <= mxcol) &&
 	   !((nc = lookat (row, nextcol)) -> flags & is_valid) &&
@@ -1679,19 +1672,19 @@ showstring (string, dirflush, hasvalue, row, col, nextcolp, mxcol, fieldlenp, r,
 
     /* Now justify and print */
     start = (dirflush&is_leftflush) ? field : field + fieldlen - slen;
-    if( dirflush & is_label )
+    if (dirflush & is_label)
 	start = field + ((slen<fwidth[col])?(fieldlen-slen)/2:0);
     last = field+fieldlen;
     fp = field;
     while (fp < start)
 	*fp++ = ' ';
-    if( *string == '\\'  && *(string+1)!= '\0') {
+    if (*string == '\\'  && *(string+1)!= '\0') {
 	char *strt;
 	strt = ++string;
 
-	while(slen--) {
+	while (slen--) {
 		*fp++ = *string++;
-		if( *string == '\0' )
+		if (*string == '\0')
 			string = strt;
 	}
     }
@@ -1714,8 +1707,7 @@ showstring (string, dirflush, hasvalue, row, col, nextcolp, mxcol, fieldlenp, r,
 }
 
 int
-etype(e)
-register struct enode *e;
+etype(register struct enode *e)
 {
     if (e == (struct enode *)0)
 	return NUM;
@@ -1727,60 +1719,57 @@ register struct enode *e;
 
     case '?':
     case IF:
-        return(etype(e->e.o.right->e.o.left));
+        return (etype(e->e.o.right->e.o.left));
 
     case 'f':
-        return(etype(e->e.o.right));
+        return (etype(e->e.o.right));
 
     case O_VAR: {
 	register struct ent *p;
 	p = e->e.v.vp;
 	if (p->expr) 
-	    return(p->flags & is_strexpr ? STR : NUM);
+	    return (p->flags & is_strexpr ? STR : NUM);
 	else if (p->label)
-	    return(STR);
+	    return (STR);
 	else
-	    return(NUM);
+	    return (NUM);
 	}
 
     default:
-	return(NUM);
+	return (NUM);
     }
 }
 
 /* return 1 if yes given, 0 otherwise */
 int
-yn_ask(msg)
-char	*msg;
+yn_ask(char *msg)
 {	char ch;
 
-	(void) move (0, 0);
-	(void) clrtoeol ();
-	(void) addstr (msg);
+	(void) move(0, 0);
+	(void) clrtoeol();
+	(void) addstr(msg);
 	(void) refresh();
 	ch = nmgetch();
-	if ( ch != 'y' && ch != 'Y' && ch != 'n' && ch != 'N' ) {
+	if (ch != 'y' && ch != 'Y' && ch != 'n' && ch != 'N') {
 		if (ch == ctl('g') || ch == ESC)
 			return(-1);
 		error("y or n response required");
 		return (-1);
 	}
 	if (ch == 'y' || ch == 'Y')
-		return(1);
+		return (1);
 	else
-		return(0);
+		return (0);
 }
 
 /* expand a ~ in a path to your home directory */
 #if !defined(MSDOS) && !defined(VMS)
 #include <pwd.h>
 #endif
-char	*
-findhome(path)
-char	*path;
+char *
+findhome(char *path)
 {
 	static	char	*HomeDir = NULL;
-	extern	char	*getenv();
 
 	if (*path == '~')
 	{	char	*pathptr;
@@ -1798,7 +1787,6 @@ char	*path;
 #if !defined(MSDOS) && !defined(VMS)
 		else
 		{	struct	passwd *pwent;
-			extern	struct	passwd *getpwnam();
 			char	*namep;
 			char	name[50];
 
@@ -1816,7 +1804,7 @@ char	*path;
 		strcat(tmppath, pathptr);
 		strcpy(path, tmppath);
 	}
-	return(path);
+	return (path);
 }
 
 #ifdef DOBACKUPS
@@ -1828,8 +1816,7 @@ char	*path;
  * return 1 if we were successful, 0 otherwise
  */
 int
-backup_file(path)
-char	*path;
+backup_file(char *path)
 {
 	struct	stat	statbuf;
 	char	fname[PATHLEN];
@@ -1867,16 +1854,16 @@ char	*path;
 #endif
 
 		if ((infd = open(path, O_RDONLY, 0)) < 0)
-			return(0);
+			return (0);
 
 		if ((outfd = open(tpath, O_TRUNC|O_WRONLY|O_CREAT,
 					statbuf.st_mode)) < 0)
-			return(0);
+			return (0);
 
 #ifdef sequent
-		while((count = read(infd, buf, statbuf.st_blksize)) > 0)
+		while ((count = read(infd, buf, statbuf.st_blksize)) > 0)
 #else
-		while((count = read(infd, buf, sizeof(buf))) > 0)
+		while ((count = read(infd, buf, sizeof(buf))) > 0)
 #endif
 		{	if (write(outfd, buf, count) != count)
 			{	count = -1;
@@ -1886,28 +1873,11 @@ char	*path;
 		close(infd);
 		close(outfd);
 
-		return((count < 0) ? 0 : 1);
+		return ((count < 0) ? 0 : 1);
 	}
 	else
 	if (errno == ENOENT)
-		return(1);
-	return(0);
+		return (1);
+	return (0);
 }
 #endif
-
-static int day_month_starts[12] = {0,31,59,90,120,151,181,212,243,273,304,334};
-
-double convert_date(d, m, y)
-int d;
-int m;
-int y;
-{
-  /* Convert to years since 1970. (or 2000, fix by 2070) */
-  if (y > 1970) y -= 1970;	/* Full year given */
-  else if (y >= 70) y -= 70;	/* Years since 1900 */
-  else y += 30;			/* Years since 2000 */
-  /* Use quarter days to compensate for leap years. */
-  return (double)((y * (365 * 4 + 1) + day_month_starts[m-1] * 4 + d * 4 - 2) *
-		  6 * 60 * 60);
-}
-
