@@ -2,13 +2,13 @@
  * Help functions for sc 
  * R. Bond, 1988
  * J. Buhrt 1990
- * $Revision: 6.21 $
+ * $Revision: 7.1 $
  */
 
 #ifdef QREF
 #include <stdio.h>
 char	*header = " Quick Reference";
-char	*revision = "$Revision: 6.21 $";
+char	*revision = "$Revision: 7.1 $";
 #else
 #include <curses.h>
 #include "sc.h"
@@ -57,6 +57,7 @@ char *toggleoptions[] = {
 #endif
 "     ^To  Toggle options. Toggle one option selected by o:",
 "          a    Recalculate automatically or on ``@'' commands.",
+"          o    Optimize expressions upon entry if enabled.",
 "          c    Current cell highlighting enable/disable.",  
 "          e    External function execution enable/disable.",
 "          l    Autolabeling defined cells enable/disable.",
@@ -87,8 +88,10 @@ char *setoptions[] = {
 ".Lp",
 #endif
 "     S  Set options.  Options include:",
+"          autocalc      Automatic recalculation.",
 "          byrows        Recalculate in row order. (default)",
 "          bycols        Recalculate in column order.",
+"          optimize      Optimize expressions upon entry. (default off)",
 "          iterations=n  Set the number of iterations allowed. (10)",
 "          tblstyle=xx   Set ``T'' output style to:",
 "                        0 (none), tex, latex, slatex, or tbl.",
@@ -128,6 +131,7 @@ char *cursor[] = {
 "     w          Forward then down to the next valid cell.",
 "     g          Go to a cell.  Cell name, range name, quoted string,",
 "                a number, 'error', or 'invalid' to specify which cell.",
+"     `,'        Go to a marked cell (see help screen e for more info.",
 (char *)0
 };
 
@@ -150,8 +154,9 @@ char *cell[] = {
 "     E    Edit the current cell's string part.",
 "     F    Assign a format to the current cell's numeric value.",
 "     x    Clear the current cell.",
-"     c    Copy the last marked cell to the current cell.",
-"     m    Mark a cell to be used as the source for ``c''",
+"     m    Followed by any lowercase letter, marks the current cell",
+"          with that letter.",
+"     c    Copy a marked cell to the current cell.",
 "     +    Increment numeric part",
 "     -    Decrement numeric part",
 "  RETURN  Enter insert mode if the input line was empty (ESC to edit)",
@@ -313,6 +318,8 @@ char *misc[] = {
 "     ^R       Redraw the screen.  Highlight cells with values but no",
 "              expressions.",
 "     ^X       Redraw the screen.  Show formulas, not values.",
+"     C        Redraw the screen with the row containing the current cell"
+"              centered.",
 "     @        Recalculate the spreadsheet.",
 "     ^V       Type, in the command line, the name of the current cell.",
 "     ^W       Type, in the command line, the current cell's expression.",
@@ -497,7 +504,9 @@ char *timef[] = {
 ".Lp",
 #endif
 "     @now              Return the time encoded in seconds since 1970.",
-"     @dts(m,d,y)       Return m/d/y encoded in seconds since 1970.",
+"     @dts(e1,e2,e3)    Return e1/e2/e3 encoded in seconds since 1970.",
+"                       Either m/d/y or y/m/d may be used (year must",
+"                       include the century), although y/m/d is preferred.",
 "     @tts(h,m,s)       Return h:m:s encoded in seconds since midnight.",
 " ",
 "     All of the following take an argument expressed in seconds:",
@@ -519,7 +528,7 @@ char *timef[] = {
 };
 
 #ifndef QREF
-static	int	pscreen();
+static	int	pscreen(char *screen[]);
 
 void
 help()
@@ -553,9 +562,8 @@ help()
     (void) clrtobot();
 }
 
-static	int
-pscreen(screen)
-char *screen[];
+static int
+pscreen(char *screen[])
 {
     int lineno;
     int dbline;
@@ -579,7 +587,7 @@ char	** pages[] = { intro, toggleoptions, setoptions, cursor, cell, vi,
 			file, row, range, misc, var, rangef, numericf, stringf,
 			finf, timef, NULL};
 
-void
+int
 main()
 {   int	lineno;
     char	***pagep = pages;
