@@ -8,7 +8,7 @@
  *              More mods Robert Bond, 12/86
  *		More mods by Alan Silverstein, 3-4/88, see list of changes.
  *		Currently supported by sequent!sawmill!buhrt (Jeff Buhrt)
- *		$Revision: 6.21 $
+ *		$Revision: 7.1 $
  *
  */
 
@@ -26,15 +26,11 @@
 #endif
 
 #include <stdio.h>
+#include <stdlib.h>
 #include "sc.h"
 
-extern	char	*getenv();
 extern	void	startdisp(), stopdisp();
 
-#ifdef SYSV3
-void exit();
-#endif
-
 #ifndef SAVENAME
 #define	SAVENAME "SC.SAVE" /* file name to use for emergency saves */
 #endif /* SAVENAME */
@@ -50,7 +46,7 @@ void exit();
 struct ent ***tbl;
 int strow = 0, stcol = 0;
 int currow = 0, curcol = 0;
-int savedrow, savedcol;
+int savedrow[26], savedcol[26];
 int FullUpdate = 0;
 int ClearScreen = 0;	/* don't try to be smart */
 int maxrow, maxcol;
@@ -88,6 +84,7 @@ int  showexpr  = 0;	/* Causes cell exprs to be displayed, highlighted */
 int  autocalc = 1 ;	/* 1 to calculate after each update */
 int  autolabel = 1;     /* If room, causes label to be created after a define*/
 int  calc_order = BYROWS;
+int  optimize = 0;	/* Causes numeric expressions to be optimized */
 int  tbl_style = 0;	/* headers for T command output */
 int  rndinfinity = 0;
 int  numeric_field = 0; /* Started the line editing with a number */
@@ -113,8 +110,7 @@ int VMS_read_raw = 0;
 
 /* return a pointer to a cell's [struct ent *], creating if needed */
 struct ent *
-lookat(row,col)
-int	row, col;
+lookat(int row, int col)
 {
     register struct ent **pp;
 
@@ -149,8 +145,7 @@ int	row, col;
  * We also use it as a last-deleted buffer for the 'p' command.
  */
 void
-free_ent(p)
-register struct ent *p;
+free_ent(register struct ent *p)
 {
     p->next = to_fix;
     to_fix = p;
@@ -180,9 +175,7 @@ flush_saved()
 char    *progname;
 
 int
-main (argc, argv)
-int argc;
-char  **argv;
+main (int argc, char  **argv)
 {
     int     inloop = 1;
     register int   c;
@@ -200,6 +193,7 @@ char  **argv;
      */
 
     int Mopt = 0;
+    int Oopt = 0;
     int Nopt = 0;
     int Copt = 0; 
     int Ropt = 0;
@@ -234,6 +228,9 @@ char  **argv;
 	    case 'm':
 		    Mopt = 1;
 		    break;
+	    case 'o':
+		    Oopt = 1;
+		    break;
 	    case 'n':
 		    Nopt = 1;
 		    break;
@@ -261,12 +258,16 @@ char  **argv;
     startdisp();
     signals();
 
-	/* setup the spreadsheet arrays, initscr() will get the screen size */
-    if (!growtbl(GROWNEW, 0, 0))
-    {	stopdisp();
+    /* setup the spreadsheet arrays, initscr() will get the screen size */
+    if (!growtbl(GROWNEW, 0, 0)) {
+     	stopdisp();
 	exit(1);
     }
 
+    /* unset all marks */
+    for (c = 0; c < 26; c++)
+	savedrow[c] = savedcol[c] = -1;
+
     /*
      * Build revision message for later use:
      */
@@ -279,11 +280,13 @@ char  **argv;
 
     if (argc > 1) {
 	(void) strcpy(curfile,argv[1]);
-	readfile (argv[1], 0);
+	readfile(argv[1], 0);
     }
 
     if (Mopt)
 	autocalc = 0;
+    if (Oopt)
+	optimize = 1;
     if (Nopt)
 	numeric = 1;
     if (Copt)
@@ -302,7 +305,7 @@ char  **argv;
 	nedistate = -1;
 	narg = 1;
 	if (edistate < 0 && linelim < 0 && autocalc && (changed || FullUpdate))
-	{    EvalAll ();
+	{    EvalAll();
 	     if (changed)		/* if EvalAll changed or was before */
 		anychanged = TRUE;
 	     changed = 0;
@@ -327,8 +330,8 @@ char  **argv;
 #endif
 	c = nmgetch();
 	getyx(stdscr, tempy, tempx);
-	(void) move (1, 0);
-	(void) clrtoeol ();
+	(void) move(1, 0);
+	(void) clrtoeol();
 	(void) move(tempy, tempx);
 /*	(void) fflush (stdout);*/
 	seenerr = 0;
@@ -341,11 +344,11 @@ char  **argv;
 	 */
 	/* if ((c < ' ') || ( c == DEL ))   how about international here ? PB */
 #if	pyr
-	   if ( iscntrl(c) || (c >= 011 && c <= 015) )	/* iscntrl broken in OSx4.1 */
+	    if(iscntrl(c) || (c >= 011 && c <= 015))	/* iscntrl broken in OSx4.1 */
 #else
-	   if (isascii(c) && (iscntrl(c) || (c == 020)) )	/* iscntrl broken in OSx4.1 */
+	    if (isascii(c) && (iscntrl(c) || (c == 020)))	/* iscntrl broken in OSx4.1 */
 #endif
-	    switch (c) {
+	    switch(c) {
 #ifdef SIGTSTP
 		case ctl('z'):
 		    (void) deraw();
@@ -362,9 +365,6 @@ char  **argv;
 		    FullUpdate++;
 		    ClearScreen++;
 		    (void) clearok(stdscr,1);
-		    /* Centering the display with cursor for middle */
-		    if(currow > (LINES-RESROW)/2)
-			strow = currow - ((LINES-RESROW)/2);
 		    break;
 		case ctl('x'):
 		    FullUpdate++;
@@ -455,35 +455,13 @@ char  **argv;
 			line[linelim] = '\0';
 			showrange = 0;
 		    }
-		    linelim = strlen (line);
+		    linelim = strlen(line);
 		    break;
 
 		case ctl('m'):
 		case ctl('j'):
 		    numeric_field = 0;
 		    write_line(ctl('m'));
-		    switch(craction) {
-		      case CRROWS:
-			if ((rowlimit >= 0) && (currow >= rowlimit)) {
-			    forwcol(1);
-			    currow = 0;
-			}
-			else {
-			    forwrow(1);
-			}
-			break;
-		      case CRCOLS:
-			if ((collimit >= 0) && (curcol >= collimit)) {
-			    forwrow(1);
-			    curcol = 0;
-			}
-			else {
-			    forwcol(1);
-			}
-			break;
-		      default:
-			break;
-		      }
 		    break;
 
 		case ctl('n'):
@@ -525,46 +503,51 @@ char  **argv;
 			    error("Automatic recalculation %sabled.",
 				autocalc ? "en":"dis");
 			    break;
+			case 'o': case 'O':
+			    optimize ^= 1;
+			    error("%sptimize expressions upon entry.",
+				optimize ? "O":"Do not o");
+			    break;
 			case 'n': case 'N':
 			    numeric = (! numeric);
-			    error ("Numeric input %sabled.",
+			    error("Numeric input %sabled.",
 				    numeric ? "en" : "dis");
 			    break;
 			case 't': case 'T':
 			    showtop = (! showtop);
-			    error ("Top line %sabled.", showtop ? "en" : "dis");
+			    error("Top line %sabled.", showtop ? "en" : "dis");
 			    break;
 			case 'c': case 'C':
 			    showcell = (! showcell);
 			    repaint(lastmx, lastmy, fwidth[lastcol]);
-			    error ("Cell highlighting %sabled.",
+			    error("Cell highlighting %sabled.",
 				    showcell ? "en" : "dis");
 			    break;
 			case 'x': case 'X':
 #if defined(VMS) || defined(MSDOS) || !defined(CRYPT_PATH)
-			    error ("Encryption not available.");
+			    error("Encryption not available.");
 #else 
 			    Crypt = (! Crypt);
-			    error ("Encryption %sabled.", Crypt? "en" : "dis");
+			    error("Encryption %sabled.", Crypt? "en" : "dis");
 #endif
 			    break;
 			case 'l': case 'L':
 			    autolabel = (! autolabel);
-			    error ("Autolabel %sabled.",
+			    error("Autolabel %sabled.",
 				   autolabel? "en" : "dis");
 			    break;
 			case '$':
 			    if (prescale == 1.0) {
-				error ("Prescale enabled.");
+				error("Prescale enabled.");
 				prescale = 0.01;
 			    } else {
 				prescale = 1.0;
-				error ("Prescale disabled.");
+				error("Prescale disabled.");
 			    }
 			    break;
 			case 'e': case 'E':
 			    extfunc = (! extfunc);
-			    error ("External functions %sabled.",
+			    error("External functions %sabled.",
 				    extfunc? "en" : "dis");
 			    break;
 			case ESC:
@@ -572,22 +555,28 @@ char  **argv;
 			    --modflg;	/* negate the modflg++ */
 			    break;
 			case 'r': case 'R':
-			    ++craction;
-			    if(craction >= 3)
-				craction = 0;
-			    switch(craction) {
-				default:
-				    craction = 0; /* fall through */
-				case 0:
-				    error("No action after new line");
-				    break;
-				case CRROWS:
-				    error("Down row after new line");
-				    break;
-				case CRCOLS:
-				    error("Right column after new line");
-				    break;
-			    }
+			    do {
+				++craction;
+				if (craction >= 3)
+				    craction = 0;
+				switch(craction) {
+				    default:
+					craction = 0; /* fall through */
+				    case 0:
+					error("No action after new line"
+                                              " (press RETURN to accept)");
+					break;
+				    case CRROWS:
+					error("Down row after new line"
+                                              " (press RETURN to accept)");
+					break;
+				    case CRCOLS:
+					error("Right column after new line"
+                                              " (press RETURN to accept)");
+					break;
+				}
+			    } while (nmgetch() != '\r');
+			    error("");
 			    break;
 			case 'z': case 'Z':
 			    rowlimit = currow;
@@ -595,7 +584,7 @@ char  **argv;
 			    error("Row and column limits set");
 			    break;
 			default:
-			    error ("Invalid toggle command");
+			    error("Invalid toggle command");
 			    --modflg;	/* negate the modflg++ */
 		    }
 		    FullUpdate++;
@@ -642,7 +631,7 @@ char  **argv;
 			char temp[100];
 
 			if (p && p -> flags & is_valid) {
-			    (void) sprintf (temp, "%.*f",
+			    (void) sprintf(temp, "%.*f",
 					precision[curcol],p -> v);
 			    ins_string(temp);
 			}
@@ -681,7 +670,7 @@ char  **argv;
 	    }
 	    write_line(c);
 
-	} else if (!numeric && ( c == '+' || c == '-' ) ) {
+	} else if (!numeric && ( c == '+' || c == '-' )) {
 	    /* increment/decrement ops */
 	    register struct ent *p = *ATBL(tbl, currow, curcol);
 	    if (!p)
@@ -692,18 +681,18 @@ char  **argv;
 	    }
 	    FullUpdate++;
 	    modflg++;
-	    if( c == '+' )
+	    if (c == '+')
 	    	p -> v += (double) arg;
 	    else
 		p -> v -= (double) arg;
 	} else
 	    /* switch on a normal command character */
-	    switch (c) {
+	    switch(c) {
 		case ':':
 		    break;	/* Be nice to vi users */
 
 		case '@':
-		    EvalAll ();
+		    EvalAll();
 		    changed = 0;
 		    anychanged = TRUE;
 		    break;
@@ -716,7 +705,7 @@ char  **argv;
 		    numeric_field = 1;
 		    (void) sprintf(line,"let %s = %c",
 				v_name(currow, curcol), c);
-		    linelim = strlen (line);
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 
@@ -725,7 +714,7 @@ char  **argv;
 			break;
 		    (void) sprintf(line,"let %s = ",
 					v_name(currow, curcol));
-		    linelim = strlen (line);
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 
@@ -760,14 +749,14 @@ char  **argv;
 		    if (modflg)
 		    {
 			(void) puts ("[No write since last change]");
-			(void) fflush (stdout);
+			(void) fflush(stdout);
 		    }
 
 		    if (!(pid = fork()))
 		    {
-			(void) signal (SIGINT, SIG_DFL);  /* reset */
+			(void) signal(SIGINT, SIG_DFL);  /* reset */
 			if(strlen(cmd))
-				(void)execl(shl,shl,"-c",cmd,(char *)0);
+				(void) execl(shl,shl,"-c",cmd,(char *)0);
 			else
 				(void) execl(shl, shl, (char *)0);
 			exit(-127);
@@ -788,7 +777,7 @@ char  **argv;
 		 */
 
 		case '/':
-		    error (
+		    error(
 "Range: x:erase v:value c:copy f:fill d:def l:lock U:unlock s:show u:undef F:fmt");
 		    (void) refresh();
 
@@ -843,24 +832,23 @@ char  **argv;
 			modflg++;
 			break;
 		    case 's':
-			if(are_ranges())
-			{
-			FILE *f;
-			int pid;
-			char px[MAXCMD] ;
-			char *pager;
+			if (are_ranges()) {
+			    FILE *f;
+			    int pid;
+			    char px[MAXCMD];
+			    char *pager;
 
-			(void) strcpy(px, "| sort | ");
-			if(!(pager = getenv("PAGER")))
+			    (void) strcpy(px, "| sort | ");
+			    if (!(pager = getenv("PAGER")))
 				pager = DFLT_PAGER;
-			(void) strcat(px,pager);
-			f = openout(px, &pid);
-			if (!f) {
-			    error("Can't open pipe to sort");
-			    break;
-			}
-			list_range(f);
-			closeout(f, pid);
+			    (void) strcat(px,pager);
+			    f = openout(px, &pid);
+			    if (!f) {
+				error("Can't open pipe to sort");
+				break;
+			    }
+			    list_range(f);
+			    closeout(f, pid);
 			}
 			else error("No ranges defined");
 			break;
@@ -873,11 +861,11 @@ char  **argv;
 		    case ESC:
 		    case ctl('g'):
 			break;
-		   default:
+		    default:
 			error("Invalid region command");
 			break;
-		   }
-		   break;
+		    }
+		    break;
 
 		/*
 		 * Row/column commands:
@@ -893,14 +881,14 @@ char  **argv;
 		    {
 			register rcqual;
 
-			if (! (rcqual = get_rcqual (c))) {
+			if (! (rcqual = get_rcqual(c))) {
 			    error ("Invalid row/column command");
 			    break;
 			}
 
 			error ("");	/* clear line */
 
-			if ( rcqual == ESC || rcqual == ctl('g'))
+			if (rcqual == ESC || rcqual == ctl('g'))
 			    break;
 
 			switch (c) {
@@ -1012,7 +1000,7 @@ char  **argv;
 				    break;
 				}
 			    }
-			} while(col_hidden[curcol] ||
+			} while (col_hidden[curcol] ||
 				!VALID_CELL(p, currow, curcol));
 		    }
 		    break;
@@ -1025,45 +1013,45 @@ char  **argv;
 		    break;
 		case '"':
 		    if (!locked_cell(currow, curcol)) {
-		       (void) sprintf (line, "label %s = \"",
-					   v_name(currow, curcol));
-		       linelim = strlen (line);
+		       (void) sprintf(line, "label %s = \"",
+				v_name(currow, curcol));
+		       linelim = strlen(line);
 		       insert_mode();
 		    }
 		    break;
 
 		case '<':
 		    if (!locked_cell(currow, curcol)) {
-		       (void) sprintf (line, "leftstring %s = \"",
+		       (void) sprintf(line, "leftstring %s = \"",
 			       v_name(currow, curcol));
-		       linelim = strlen (line);
+		       linelim = strlen(line);
 		       insert_mode();
 		    }
 		    break;
 
 		case '>':
 		    if (!locked_cell(currow, curcol)) {
-		       (void) sprintf (line, "rightstring %s = \"",
+		       (void) sprintf(line, "rightstring %s = \"",
 			      v_name(currow, curcol));
-		       linelim = strlen (line);
+		       linelim = strlen(line);
 		       insert_mode();
 		    }
 		    break;
 		case 'e':
 		    if (!locked_cell(currow, curcol)) {
-		       editv (currow, curcol);
+		       editv(currow, curcol);
 		       edit_mode();
 		    }
 		    break;
 		case 'E':
 		    if (!locked_cell(currow, curcol)) {
-		       edits (currow, curcol);
+		       edits(currow, curcol);
 		       edit_mode();
 		    }
 		    break;
 		case 'f':
 		    if (arg == 1)
-			(void) sprintf (line, "format [for column] %s ",
+			(void) sprintf(line, "format [for column] %s ",
 				coltoa(curcol));
 		    else {
 			(void) sprintf(line, "format [for columns] %s:",
@@ -1073,18 +1061,17 @@ char  **argv;
 		    }
 		    error("Current format is %d %d %d",
 			fwidth[curcol],precision[curcol],realfmt[curcol]);
-		    linelim = strlen (line);
+		    linelim = strlen(line);
 		    insert_mode();
 		    break;
 		case 'F': {
 		    register struct ent *p = *ATBL(tbl, currow, curcol);
-		    if (p && p->format)
-		    {	(void) sprintf(line, "fmt [format] %s \"%s",
-				   v_name(currow, curcol), p->format);
+		    if (p && p->format) {
+			(void) sprintf(line, "fmt [format] %s \"%s",
+			    v_name(currow, curcol), p->format);
 			edit_mode();
-		    }
-		    else
-		    {	(void) sprintf(line, "fmt [format] %s \"",
+		    } else {
+			(void) sprintf(line, "fmt [format] %s \"",
 				   v_name(currow, curcol));
 			insert_mode();
 		    }
@@ -1209,14 +1196,21 @@ char  **argv;
 		case KEY_NPAGE:			/* page precedente */
 #endif
 		case 'J':
-			forwrow(LINES-RESROW-(currow-strow)+1);
-			break;
+		    forwrow((LINES-RESROW)/2);
+		    strow = strow + (LINES-RESROW)/2;
+		    FullUpdate++;
+		    break;
 #ifdef	KEY_PPAGE
 		case KEY_PPAGE:			/* page suivante */
 #endif
 		case 'K':
-			backrow((currow-strow+1)+3);
-			break;
+		    backrow((LINES-RESROW)/2);
+		    if (strow > (LINES-RESROW)/2)
+			strow = strow - (LINES-RESROW)/2;
+		    else
+			strow = 0;
+		    FullUpdate++;
+		    break;
 #ifdef KEY_HOME
 		case KEY_HOME:
 			currow = 0;
@@ -1232,24 +1226,76 @@ char  **argv;
 		    forwcol(arg);
 		    break;
 		case 'm':
-		    savedrow = currow;
-		    savedcol = curcol;
-		    break;
-		case 'c': {
-		    register struct ent *p = *ATBL(tbl, savedrow, savedcol);
-		    register c1;
-		    register struct ent *n;
-		    if (!p)
+		    error("Mark cell (select a-z)");
+		    if ((c=nmgetch()) == ESC || c == ctl('g')) {
+			error("");
 			break;
-		    FullUpdate++;
-		    modflg++;
-		    for (c1 = curcol; arg-- && c1 < maxcols; c1++) {
-			n = lookat (currow, c1);
-			(void) clearent(n);
-			copyent( n, p, currow - savedrow, c1 - savedcol);
 		    }
+		    if ((c -= 'a') < 0 || c > 25) {
+			error("Invalid character for mark (must be a-z)");
+			break;
+		    }
+		    error("");
+		    savedrow[c] = currow;
+		    savedcol[c] = curcol;
+		    break;
+		case 'c':
+		    error("Copy marked cell (select a-z)");
+		    if ((c=nmgetch()) == ESC || c == ctl('g')) {
+			error("");
+			break;
+		    }
+		    if ((c -= 'a') < 0 || c > 25) {
+			error("Invalid character for mark (must be a-z)");
+			break;
+		    }
+		    if (savedrow[c] == -1) {
+			error("Mark not set");
+			break;
+		    }
+		    error("");
+		    {
+			register struct ent *p = *ATBL(tbl, savedrow[c], savedcol[c]);
+			register c1;
+			register struct ent *n;
+			if (!p)
+			    break;
+			FullUpdate++;
+			modflg++;
+			for (c1 = curcol; arg-- && c1 < maxcols; c1++) {
+			    n = lookat (currow, c1);
+			    (void) clearent(n);
+			    copyent( n, p, currow - savedrow[c], c1 - savedcol[c]);
+			}
+		    break;
+		    }
+		case '`': case '\'':
+		    error("Go to marked cell (select a-z)");
+		    if ((c=nmgetch()) == ESC || c == ctl('g')) {
+			error("");
+			break;
+		    }
+		    if ((c -= 'a') < 0 || c > 25) {
+			error("Invalid character for mark (must be a-z)");
+			break;
+		    }
+		    if (savedrow[c] == -1) {
+			error("Mark not set");
+			break;
+		    }
+		    error("");
+		    currow = savedrow[c];
+		    curcol = savedcol[c];
+		    FullUpdate++;
+		    break;
+		case 'C':
+		    FullUpdate++;
+		    ClearScreen++;
+		    (void) clearok(stdscr,1);
+		    /* Center the display with cursor for middle */
+		    if(currow > (LINES-RESROW)/2)
+			strow = currow - ((LINES-RESROW)/2);
 		    break;
-		}
 		default:
 		    if ((toascii(c)) != c)
 			error ("Weird character, decimal %d\n",
@@ -1296,16 +1342,14 @@ showdr()
 
 /* set the calculation order */
 void
-setorder(i)
-int i;
+setorder(int i)
 {
 	if((i == BYROWS)||(i == BYCOLS))
 	    calc_order = i;
 }
 
 void
-setauto(i)
-int i;
+setauto(int i)
 {
 	autocalc = i;
 }
@@ -1397,8 +1441,7 @@ diesave()
 
 /* check if tbl was modified and ask to save */
 int
-modcheck(endstr)
-char *endstr;
+modcheck(char *endstr)
 {
     if (modflg && curfile[0]) {
 	int	yn_ans;
@@ -1425,8 +1468,7 @@ char *endstr;
 
 /* Returns 1 if cell is locked, 0 otherwise */
 int
-locked_cell (r, c)
-	int r, c;
+locked_cell(int r, int c)
 {
 	struct ent *p = *ATBL(tbl, r, c);
 	if (p && (p->flags & is_locked)) {
@@ -1438,8 +1480,7 @@ locked_cell (r, c)
 
 /* Check if area contains locked cells */
 int
-any_locked_cells(r1, c1, r2, c2)
-	int r1, c1, r2, c2 ;
+any_locked_cells(int r1, int c1, int r2, int c2)
 {
 	int r, c;
 	struct ent *p ;
@@ -1447,7 +1488,7 @@ any_locked_cells(r1, c1, r2, c2)
 	for (r=r1; r<=r2; r++)
 		for (c=c1; c<=c2; c++) {
 			p = *ATBL(tbl, r, c);
-			if (p && (p->flags & is_locked))
+			if (p && (p->flags&is_locked))
 				return(1);
 		}
 	return(0);
